/*=============================================================================
 *
 *  Professor Douglas Lyon                                    Kirill Golubev
 *  CS588 Computer Graphics in Java                           Hw #7
 *                                                            incorporates all
 *                                                            previous hwmks
 *                                                            04/22/99
 *
 *=============================================================================
 */

/*=============================================================================
    Uses two-dimentional rendering application and creates a 3D graphics
    editor.  Main module.
 *=============================================================================
 */

package gui;

import java.awt.*;
import java.awt.event.*;

/*=============================================================================
    Main module, gui interface.
 *=============================================================================
 */

public class GraphEditor3D extends XformFrame implements ActionListener {


    //
    //  Setting up menus for Perpendicular and Planar Geometric Projections.
    //
    MenuBar graphEd_menuBar = new MenuBar();

    Menu PlanGeoPrjMenu = new Menu("Planar Geom. Projections");
    Menu ParallMenu = new Menu("Parallel");
    MenuItem orthogr_mItem = addMenuItem(ParallMenu, "Orthographic...");

    Menu AxonometrMenu = new Menu("Axonometric");
    MenuItem trimetric_mi = addMenuItem(AxonometrMenu, "Trimetric...");
    MenuItem dimetric_mi = addMenuItem(AxonometrMenu, "Dimetric...");
    MenuItem isometric_mi = addMenuItem(AxonometrMenu, "Isometric...");

    Menu ObliqueMenu = new Menu("Oblique");
    MenuItem cavalier_mi = addMenuItem(ObliqueMenu, "Cavalier...");
    MenuItem cabinet_mi = addMenuItem(ObliqueMenu, "Cabinet...");

    Menu PerspectiveMenu = new Menu("Perspective");
    MenuItem one_point_mi = addMenuItem(PerspectiveMenu, "1 point");
    MenuItem two_point_mi = addMenuItem(PerspectiveMenu, "2 point");
    MenuItem three_point_mi = addMenuItem(PerspectiveMenu, "3 point");


    //
    //  Modal Dialog box, displays a string supplied as an argument to the constructor.
    //
    class MessageDialog extends Dialog implements ActionListener {

        private Label message;
        private Button close;

        MessageDialog(Frame parent, String title, String messageString) {

            super(parent);
            setSize(100, 100);
            setResizable(false);

            message = new Label(messageString, Label.CENTER);
            add(message, BorderLayout.CENTER);

            close = new Button("Close");
            close.addActionListener(this);
            add(close, BorderLayout.SOUTH);
            setVisible(true);
        }

        public void actionPerformed(ActionEvent e) {
            setVisible(false);
        }
    }


    public void actionPerformed(ActionEvent e) {

        if (e.getSource() == trimetric_mi) {
            PhiTheta_UserInterface ptun = new PhiTheta_UserInterface();
            return;
        }
        if (e.getSource() == one_point_mi) {
            VanishPnt_UserInterface vpui_single = new VanishPnt_UserInterface("single_point");
        }
        if (e.getSource() == two_point_mi) {
            VanishPnt_UserInterface vpui_two = new VanishPnt_UserInterface("two_point");
        }
        if (e.getSource() == three_point_mi) {
            VanishPnt_UserInterface vpui_three = new VanishPnt_UserInterface("three_point");
        }
    }


    GraphEditor3D(String title) {

        super(title);

        ParallMenu.add(AxonometrMenu);
        ParallMenu.add(ObliqueMenu);
        PlanGeoPrjMenu.add(ParallMenu);
        PlanGeoPrjMenu.add(PerspectiveMenu);
        PlanGeoPrjMenu.addSeparator();
        graphEd_menuBar.add(PlanGeoPrjMenu);

        setMenuBar(graphEd_menuBar);
        setSize(300, 300);
        setVisible(true);
    }


    public static void main(String args[]) {
        GraphEditor3D ge3D = new GraphEditor3D("GraphEditor3D");
    }

//
//  User Interface to accept two angles, phi and theta
//
    class PhiTheta_UserInterface extends Frame implements ItemListener, ActionListener,
            AdjustmentListener {

        private Label trimetric_angels_label = new Label("TRIMETRIC ANGLES");
        private Label view_label = new Label("View");
        private Choice view_choice = new Choice();
        private String[] view = {"FRONT", "BACK", "TOP", "BOTTOM", "LEFT", "RIGHT"};

        private int phi_angle, theta_angle;
        private int scrbar_initpos = phi_angle = theta_angle = 50;
        private String selected_view = view[0];

        private Label phi_angle_label = new Label(" Phi Angle");
        private TextField phi_angle_txtfld = new TextField(3);
        private Scrollbar phi_angle_scrbar = new Scrollbar(Scrollbar.VERTICAL, scrbar_initpos, 1, 0, 101);

        private Label theta_angle_label = new Label("Theta Angle");
        private TextField theta_angle_txtfld = new TextField(3);
        private Scrollbar theta_angle_scrbar = new Scrollbar(Scrollbar.VERTICAL, scrbar_initpos, 1, 0, 101);

        private Button show_buttn = new Button("OK");
        private Button cancel_buttn = new Button("Cancel");

        private GridBagLayout gr_bag_layout = new GridBagLayout();
        private GridBagConstraints grbConstraints = new GridBagConstraints();


        public void itemStateChanged(ItemEvent e) {

            selected_view = view_choice.getSelectedItem();
        }


        public void actionPerformed(ActionEvent e) {

            if (e.getSource() == show_buttn) {
                System.out.println("phi   angle is " + phi_angle);
                System.out.println("theta angle is " + theta_angle);
                System.out.println("view  is       " + selected_view);

                return;
            }
            if (e.getSource() == cancel_buttn) {
                setVisible(false);

                return;
            }
        }


        public void adjustmentValueChanged(AdjustmentEvent e) {

            if (e.getSource() == phi_angle_scrbar) {
                int input_value = e.getValue();
                phi_angle = input_value;
                phi_angle_txtfld.setText("" + input_value);

                return;
            }
            if (e.getSource() == theta_angle_scrbar) {
                int input_value = e.getValue();
                theta_angle = input_value;
                theta_angle_txtfld.setText("" + input_value);

                return;
            }
        }


        PhiTheta_UserInterface() {

            setLayout(gr_bag_layout);
            addComponent(trimetric_angels_label, 0, 0, 6, 1);

            //
            //  Setting up the choice views.
            //
            addComponent(view_label, 2, 1, 1, 1);
            for (int i = 0; i < view.length; i++) {
                view_choice.addItem(view[i]);
            }
            addComponent(view_choice, 3, 1, 3, 1);
            view_choice.addItemListener(this);

            //
            //  Setting up components to accept phi angle
            //
            addComponent(phi_angle_label, 0, 2, 2, 1);
            addComponent(phi_angle_txtfld, 2, 2, 1, 1);
            phi_angle_txtfld.setEditable(false);
            phi_angle_txtfld.setText("" + scrbar_initpos);
            addComponent(phi_angle_scrbar, 3, 2, 1, 1);
            phi_angle_scrbar.addAdjustmentListener(this);
            addComponent((new Label("radians")), 4, 2, 1, 1);

            //
            //  Setting up components to accept theta angle
            //
            addComponent(theta_angle_label, 0, 3, 2, 1);
            addComponent(theta_angle_txtfld, 2, 3, 1, 1);
            theta_angle_txtfld.setEditable(false);
            theta_angle_txtfld.setText("" + scrbar_initpos);
            addComponent(theta_angle_scrbar, 3, 3, 1, 1);
            theta_angle_scrbar.addAdjustmentListener(this);
            addComponent((new Label("radians")), 4, 3, 1, 1);

            addComponent(show_buttn, 1, 4, 2, 1);
            show_buttn.addActionListener(this);
            addComponent(cancel_buttn, 3, 4, 2, 1);
            cancel_buttn.addActionListener(this);

            setSize(280, 200);
            setVisible(true);
        }


        //
        // addComponent is programmer defined
        //
        private void addComponent(Component c, int column, int row, int width, int height) {

            grbConstraints.gridx = column;
            grbConstraints.gridy = row;
            grbConstraints.gridwidth = width;
            grbConstraints.gridheight = height;
            gr_bag_layout.setConstraints(c, grbConstraints);
            add(c);
        }


        //public static void main(String args[]) {
        //   PhiTheta_UserInterface trinter = new PhiTheta_UserInterface();
        //}
    }





//
//  User Interface to accept the coordinates of the vanishing point.
//
    class VanishPnt_UserInterface extends Frame implements ItemListener, ActionListener,
            AdjustmentListener {

        private Label vanish_pnt_label = new Label("COORDINATES OF THE VANISHING POINT");
        private Label single_pnt_label = new Label("Single Point Perspective Transform");
        private Label two_pnt_label = new Label("Two Point Perspective Transform");
        private Label three_pnt_label = new Label("Three Point Perspective Transform");

        private Label view_label = new Label("View");
        private Choice view_choice = new Choice();
        private String[] view = {"FRONT", "BACK", "TOP", "BOTTOM", "LEFT", "RIGHT"};

        //
        // Initialize all the global variables
        //
        private int x_coord, y_coord, z_coord;
        private int scrbar_initpos = x_coord = y_coord = z_coord = 0;
        private String selected_view = view[0];

        private Label x_coord_label = new Label("    X ");
        private TextField x_coord_txtfld = new TextField(3);
        private Scrollbar x_coord_scrbar = new Scrollbar(Scrollbar.VERTICAL, scrbar_initpos, 1, 0, 301);

        private Label y_coord_label = new Label("  Y");
        private TextField y_coord_txtfld = new TextField(3);
        private Scrollbar y_coord_scrbar = new Scrollbar(Scrollbar.VERTICAL, scrbar_initpos, 1, 0, 301);

        private Label z_coord_label = new Label("  Z");
        private TextField z_coord_txtfld = new TextField(3);
        private Scrollbar z_coord_scrbar = new Scrollbar(Scrollbar.VERTICAL, scrbar_initpos, 1, 0, 301);

        private Button show_buttn = new Button("Show");
        private Button cancel_buttn = new Button("Cancel");

        private GridBagLayout gr_bag_layout = new GridBagLayout();
        private GridBagConstraints grbConstraints = new GridBagConstraints();


        public void itemStateChanged(ItemEvent e) {

            selected_view = view_choice.getSelectedItem();
        }


        public void actionPerformed(ActionEvent e) {

            if (e.getSource() == show_buttn) {
                System.out.println("x coordinate is " + x_coord);
                System.out.println("y coordinate is " + y_coord);
                System.out.println("z coordinate is " + z_coord);
                System.out.println("view  is        " + selected_view);

                return;
            }
            if (e.getSource() == cancel_buttn) {
                setVisible(false);

                return;
            }
        }


        public void adjustmentValueChanged(AdjustmentEvent e) {

            if (e.getSource() == x_coord_scrbar) {
                int input_value = e.getValue();
                x_coord = input_value;
                x_coord_txtfld.setText("" + input_value);

                return;
            }
            if (e.getSource() == y_coord_scrbar) {
                int input_value = e.getValue();
                y_coord = input_value;
                y_coord_txtfld.setText("" + input_value);

                return;
            }
            if (e.getSource() == z_coord_scrbar) {
                int input_value = e.getValue();
                z_coord = input_value;
                z_coord_txtfld.setText("" + input_value);

                return;
            }
        }

        //
        //  Arguments to the constructor:  single_point, two_point, three_point.
        //
        VanishPnt_UserInterface(String what_point) {

            setLayout(gr_bag_layout);
            addComponent(vanish_pnt_label, 0, 0, 6, 1);

            if (what_point.equals("single_point")) addComponent(single_pnt_label, 0, 1, 6, 1);
            if (what_point.equals("two_point")) addComponent(two_pnt_label, 0, 1, 6, 1);
            if (what_point.equals("three_point")) addComponent(three_pnt_label, 0, 1, 6, 1);

            //
            //  Setting up the choice views.
            //
            addComponent(view_label, 2, 2, 1, 1);
            for (int i = 0; i < view.length; i++) {
                view_choice.addItem(view[i]);
            }
            addComponent(view_choice, 3, 2, 2, 1);
            view_choice.addItemListener(this);

            //
            //  Setting up components to accept x coordinate.
            //
            addComponent(x_coord_label, 1, 3, 1, 1);
            addComponent(x_coord_txtfld, 2, 3, 1, 1);
            x_coord_txtfld.setEditable(false);
            x_coord_txtfld.setText("" + scrbar_initpos);

            if (what_point.equals("two_point") || what_point.equals("three_point")) {
                addComponent(x_coord_scrbar, 3, 3, 1, 1);
                x_coord_scrbar.addAdjustmentListener(this);
            }

            //
            //  Setting up components to accept y coordinate.
            //
            addComponent(y_coord_label, 1, 4, 1, 1);
            addComponent(y_coord_txtfld, 2, 4, 1, 1);
            y_coord_txtfld.setEditable(false);
            y_coord_txtfld.setText("" + scrbar_initpos);

            if (what_point.equals("two_point") || what_point.equals("three_point")) {
                addComponent(y_coord_scrbar, 3, 4, 1, 1);
                y_coord_scrbar.addAdjustmentListener(this);
            }

            //
            //  Setting up components to accept z coordinate.
            //
            addComponent(z_coord_label, 1, 5, 1, 1);
            addComponent(z_coord_txtfld, 2, 5, 1, 1);
            z_coord_txtfld.setEditable(false);
            z_coord_txtfld.setText("" + scrbar_initpos);

            if (what_point.equals("single_point") || what_point.equals("three_point")) {
                addComponent(z_coord_scrbar, 3, 5, 1, 1);
                z_coord_scrbar.addAdjustmentListener(this);
            }


            addComponent(show_buttn, 1, 6, 2, 1);
            show_buttn.addActionListener(this);
            addComponent(cancel_buttn, 3, 6, 2, 1);
            cancel_buttn.addActionListener(this);

            setSize(280, 300);
            setVisible(true);
        }


        //
        // addComponent is programmer defined
        //
        private void addComponent(Component c, int column, int row, int width, int height) {

            grbConstraints.gridx = column;
            grbConstraints.gridy = row;
            grbConstraints.gridwidth = width;
            grbConstraints.gridwidth = width;
            grbConstraints.gridheight = height;
            gr_bag_layout.setConstraints(c, grbConstraints);
            add(c);
        }

        //public static void main(String args[]) {
        //  VanishPnt_UserInterface  vpui_single = new VanishPnt_UserInterface("single_point");
        //  VanishPnt_UserInterface  vpui_two    = new VanishPnt_UserInterface("two_point");
        //  VanishPnt_UserInterface  vpui_three  = new VanishPnt_UserInterface("three_point");
        //}
    }


} // Main class ends.